@@ -7,19 +7,14 @@
  *
  */
 
-/**
- * SECTION:gupnp-device-info
- * @short_description: Base abstract class for querying device information.
- *
- * The #GUPnPDeviceInfo base abstract class provides methods for querying
- * device information.
- */
+
 
 #include <config.h>
 #include <string.h>
 
-#include "gupnp-device-info.h"
+#include "gupnp-context-private.h"
 #include "gupnp-device-info-private.h"
+#include "gupnp-device-info.h"
 #include "gupnp-resource-factory-private.h"
 #include "xml-util.h"
 
@@ -31,7 +26,7 @@ struct _GUPnPDeviceInfoPrivate {
         char *udn;
         char *device_type;
 
-        SoupURI *url_base;
+        GUri *url_base;
 
         GUPnPXMLDoc *doc;
 
@@ -39,6 +34,30 @@ struct _GUPnPDeviceInfoPrivate {
 };
 typedef struct _GUPnPDeviceInfoPrivate GUPnPDeviceInfoPrivate;
 
+struct _GetIconData {
+        char *mime;
+        int depth;
+        int width;
+        int height;
+};
+typedef struct _GetIconData GetIconData;
+
+static void
+get_icon_data_free (GetIconData *data)
+{
+        g_clear_pointer (&data->mime, g_free);
+        g_free (data);
+}
+
+/**
+ * GUPnPDeviceInfo:
+ *
+ * Device information shared by local and remote devices
+ *
+ * This class aggregates the information that is shared between remote and local
+ * devices.
+ */
+
 G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (GUPnPDeviceInfo,
                                      gupnp_device_info,
                                      G_TYPE_OBJECT)
@@ -177,7 +196,7 @@ gupnp_device_info_finalize (GObject *object)
         g_free (priv->udn);
         g_free (priv->device_type);
 
-        g_clear_pointer (&priv->url_base, soup_uri_free);
+        g_clear_pointer (&priv->url_base, g_uri_unref);
 
         G_OBJECT_CLASS (gupnp_device_info_parent_class)->finalize (object);
 }
@@ -195,120 +214,104 @@ gupnp_device_info_class_init (GUPnPDeviceInfoClass *klass)
         object_class->finalize     = gupnp_device_info_finalize;
 
         /**
-         * GUPnPDeviceInfo:resource-factory:
+         * GUPnPDeviceInfo:resource-factory:(attributes org.gtk.Property.get=gupnp_device_info_get_resource_factory):
          *
          * The resource factory to use. Set to NULL for default factory.
          **/
-        g_object_class_install_property
-                (object_class,
-                 PROP_RESOURCE_FACTORY,
-                 g_param_spec_object ("resource-factory",
-                                      "Resource Factory",
-                                      "The resource factory to use",
-                                      GUPNP_TYPE_RESOURCE_FACTORY,
-                                      G_PARAM_READWRITE |
-                                      G_PARAM_CONSTRUCT_ONLY |
-                                      G_PARAM_STATIC_NAME |
-                                      G_PARAM_STATIC_NICK |
-                                      G_PARAM_STATIC_BLURB));
+        g_object_class_install_property (
+                object_class,
+                PROP_RESOURCE_FACTORY,
+                g_param_spec_object ("resource-factory",
+                                     "Resource Factory",
+                                     "The resource factory to use",
+                                     GUPNP_TYPE_RESOURCE_FACTORY,
+                                     G_PARAM_READWRITE |
+                                             G_PARAM_CONSTRUCT_ONLY |
+                                             G_PARAM_STATIC_STRINGS));
 
         /**
-         * GUPnPDeviceInfo:context:
+         * GUPnPDeviceInfo:context:(attributes org.gtk.Property.get=gupnp_device_info_get_context):
          *
          * The #GUPnPContext to use.
          **/
-        g_object_class_install_property
-                (object_class,
-                 PROP_CONTEXT,
-                 g_param_spec_object ("context",
-                                      "Context",
-                                      "The GUPnPContext",
-                                      GUPNP_TYPE_CONTEXT,
-                                      G_PARAM_READWRITE |
-                                      G_PARAM_CONSTRUCT_ONLY |
-                                      G_PARAM_STATIC_NAME |
-                                      G_PARAM_STATIC_NICK |
-                                      G_PARAM_STATIC_BLURB));
+        g_object_class_install_property (
+                object_class,
+                PROP_CONTEXT,
+                g_param_spec_object ("context",
+                                     "Context",
+                                     "The GUPnPContext",
+                                     GUPNP_TYPE_CONTEXT,
+                                     G_PARAM_READWRITE |
+                                             G_PARAM_CONSTRUCT_ONLY |
+                                             G_PARAM_STATIC_STRINGS));
 
         /**
-         * GUPnPDeviceInfo:location:
+         * GUPnPDeviceInfo:location:(attributes org.gtk.Property.get=gupnp_device_info_get_location):
          *
          * The location of the device description file.
          **/
-        g_object_class_install_property
-                (object_class,
-                 PROP_LOCATION,
-                 g_param_spec_string ("location",
-                                      "Location",
-                                      "The location of the device description "
-                                      "file",
-                                      NULL,
-                                      G_PARAM_READWRITE |
-                                      G_PARAM_CONSTRUCT |
-                                      G_PARAM_STATIC_NAME |
-                                      G_PARAM_STATIC_NICK |
-                                      G_PARAM_STATIC_BLURB));
+        g_object_class_install_property (
+                object_class,
+                PROP_LOCATION,
+                g_param_spec_string ("location",
+                                     "Location",
+                                     "The location of the device description "
+                                     "file",
+                                     NULL,
+                                     G_PARAM_READWRITE | G_PARAM_CONSTRUCT |
+                                             G_PARAM_STATIC_STRINGS));
 
         /**
-         * GUPnPDeviceInfo:udn:
+         * GUPnPDeviceInfo:udn:(attributes org.gtk.Property.get=gupnp_device_info_get_udn):
          *
          * The UDN of this device.
          **/
-        g_object_class_install_property
-                (object_class,
-                 PROP_UDN,
-                 g_param_spec_string ("udn",
-                                      "UDN",
-                                      "The UDN",
-                                      NULL,
-                                      G_PARAM_READWRITE |
-                                      G_PARAM_CONSTRUCT_ONLY |
-                                      G_PARAM_STATIC_NAME |
-                                      G_PARAM_STATIC_NICK |
-                                      G_PARAM_STATIC_BLURB));
-
-        /**
-         * GUPnPDeviceInfo:device-type:
-         *
-         * The device type.
-         **/
-        g_object_class_install_property
-                (object_class,
-                 PROP_DEVICE_TYPE,
-                 g_param_spec_string ("device-type",
-                                      "Device type",
-                                      "The device type",
-                                      NULL,
-                                      G_PARAM_READWRITE |
-                                      G_PARAM_CONSTRUCT_ONLY |
-                                      G_PARAM_STATIC_NAME |
-                                      G_PARAM_STATIC_NICK |
-                                      G_PARAM_STATIC_BLURB));
-
-        /**
-         * GUPnPDeviceInfo:url-base:
-         *
-         * The URL base (#SoupURI).
-         **/
-        g_object_class_install_property
-                (object_class,
-                 PROP_URL_BASE,
-                 g_param_spec_boxed ("url-base",
-                                     "URL base",
-                                     "The URL base",
-                                     SOUP_TYPE_URI,
+        g_object_class_install_property (
+                object_class,
+                PROP_UDN,
+                g_param_spec_string ("udn",
+                                     "UDN",
+                                     "The Unique Device Name",
+                                     NULL,
                                      G_PARAM_READWRITE |
-                                     G_PARAM_CONSTRUCT |
-                                     G_PARAM_STATIC_NAME |
-                                     G_PARAM_STATIC_NICK |
-                                     G_PARAM_STATIC_BLURB));
+                                             G_PARAM_CONSTRUCT_ONLY |
+                                             G_PARAM_STATIC_STRINGS));
+
+        /**
+         * GUPnPDeviceInfo:device-type:(attributes org.gtk.Property.get=gupnp_device_info_get_device_type):
+         *
+         * The device type, e.g. `urn:schemas-upnp-org:device:InternetGatewayDevice:1`
+         **/
+        g_object_class_install_property (
+                object_class,
+                PROP_DEVICE_TYPE,
+                g_param_spec_string ("device-type",
+                                     "Device type",
+                                     "The device type",
+                                     NULL,
+                                     G_PARAM_READWRITE |
+                                             G_PARAM_CONSTRUCT_ONLY |
+                                             G_PARAM_STATIC_STRINGS));
+
+        /**
+         * GUPnPDeviceInfo:url-base:(attributes org.gtk.Property.get=gupnp_device_info_get_url_base):
+         *
+         * The URL base (#GUri).
+         **/
+        g_object_class_install_property (
+                object_class,
+                PROP_URL_BASE,
+                g_param_spec_boxed ("url-base",
+                                    "URL base",
+                                    "The URL base",
+                                    G_TYPE_URI,
+                                    G_PARAM_READWRITE | G_PARAM_CONSTRUCT |
+                                            G_PARAM_STATIC_STRINGS));
 
         /**
          * GUPnPDeviceInfo:document:
          *
-         * Private property.
-         *
-         * Stability: Private
+         * The description document
          **/
         g_object_class_install_property (
                 object_class,
@@ -319,9 +322,7 @@ gupnp_device_info_class_init (GUPnPDeviceInfoClass *klass)
                                      "device",
                                      GUPNP_TYPE_XML_DOC,
                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT |
-                                             G_PARAM_STATIC_NAME |
-                                             G_PARAM_STATIC_NICK |
-                                             G_PARAM_STATIC_BLURB));
+                                             G_PARAM_STATIC_STRINGS));
 
         /**
          * GUPnPDeviceInfo:element:
@@ -330,22 +331,74 @@ gupnp_device_info_class_init (GUPnPDeviceInfoClass *klass)
          *
          * Stability: Private
          **/
-        g_object_class_install_property
-                (object_class,
-                 PROP_ELEMENT,
-                 g_param_spec_pointer ("element",
-                                       "Element",
-                                       "The XML element related to this "
-                                       "device",
-                                       G_PARAM_WRITABLE |
-                                       G_PARAM_CONSTRUCT |
-                                       G_PARAM_STATIC_NAME |
-                                       G_PARAM_STATIC_NICK |
-                                       G_PARAM_STATIC_BLURB));
+        g_object_class_install_property (
+                object_class,
+                PROP_ELEMENT,
+                g_param_spec_pointer ("element",
+                                      "Element",
+                                      "The XML element related to this "
+                                      "device",
+                                      G_PARAM_WRITABLE | G_PARAM_CONSTRUCT |
+                                              G_PARAM_STATIC_STRINGS));
 }
 
 /**
- * gupnp_device_info_get_resource_factory:
+ * gupnp_device_info_get_element:
+ * @info: a #GUPnPDeviceInfo
+ *
+ * Returns: the xmlNode associated with this device info
+ */
+xmlNode *
+gupnp_device_info_get_element (GUPnPDeviceInfo *info)
+{
+        g_return_val_if_fail (GUPNP_IS_DEVICE_INFO (info), NULL);
+        GUPnPDeviceInfoClass *class = GUPNP_DEVICE_INFO_GET_CLASS (info);
+
+        g_return_val_if_fail (class->get_element, NULL);
+
+        return class->get_element (info);
+}
+
+/**
+ * gupnp_device_info_create_device_instance:
+ * @info: a #GUPnPDeviceInfo
+ *
+ * Returns: the xmlNode associated with this device info
+ */
+GUPnPDeviceInfo *
+gupnp_device_info_create_device_instance (GUPnPDeviceInfo *info,
+                                          xmlNode *element)
+{
+        g_return_val_if_fail (GUPNP_IS_DEVICE_INFO (info), NULL);
+        GUPnPDeviceInfoClass *class = GUPNP_DEVICE_INFO_GET_CLASS (info);
+
+        g_return_val_if_fail (class->create_device_instance, NULL);
+
+        return class->create_device_instance (info, element);
+
+}
+
+/**
+ * gupnp_device_info_create_service_instance:
+ * @info: a #GUPnPDeviceInfo
+ *
+ * Returns: the xmlNode associated with this device info
+ */
+GUPnPServiceInfo *
+gupnp_device_info_create_service_instance (GUPnPDeviceInfo *info,
+                                           xmlNode *element)
+{
+        g_return_val_if_fail (GUPNP_IS_DEVICE_INFO (info), NULL);
+        GUPnPDeviceInfoClass *class = GUPNP_DEVICE_INFO_GET_CLASS (info);
+
+        g_return_val_if_fail (class->create_service_instance, NULL);
+
+        return class->create_service_instance (info, element);
+
+}
+
+/**
+ * gupnp_device_info_get_resource_factory:(attributes org.gtk.Method.get_property=resource-factory):
  * @device_info: A #GUPnPDeviceInfo
  *
  * Get the #GUPnPResourceFactory used by the @device_info.
@@ -365,12 +418,12 @@ gupnp_device_info_get_resource_factory (GUPnPDeviceInfo *info)
 }
 
 /**
- * gupnp_device_info_get_context:
+ * gupnp_device_info_get_context:(attributes org.gtk.Method.get_property=context):
  * @info: A #GUPnPDeviceInfo
  *
  * Get the associated #GUPnPContext.
  *
- * Returns: (transfer none): A #GUPnPContext.
+ * Returns: (transfer none): The #GUPnPContext the devices is operating on.
  **/
 GUPnPContext *
 gupnp_device_info_get_context (GUPnPDeviceInfo *info)
@@ -388,9 +441,9 @@ gupnp_device_info_get_context (GUPnPDeviceInfo *info)
  * gupnp_device_info_get_location:
  * @info: A #GUPnPDeviceInfo
  *
- * Get the location of the device description file.
+ * Get the URL of the device file
  *
- * Returns: A constant string.
+ * Returns: A s
  **/
 const char *
 gupnp_device_info_get_location (GUPnPDeviceInfo *info)
@@ -405,14 +458,14 @@ gupnp_device_info_get_location (GUPnPDeviceInfo *info)
 }
 
 /**
- * gupnp_device_info_get_url_base:
+ * gupnp_device_info_get_url_base:(attributes org.gtk.Method.get_property=url-base):
  * @info: A #GUPnPDeviceInfo
  *
  * Get the URL base of this device.
  *
  * Returns: A #SoupURI.
  **/
-const SoupURI *
+const GUri *
 gupnp_device_info_get_url_base (GUPnPDeviceInfo *info)
 {
         GUPnPDeviceInfoPrivate *priv;
@@ -425,7 +478,7 @@ gupnp_device_info_get_url_base (GUPnPDeviceInfo *info)
 }
 
 /**
- * gupnp_device_info_get_udn:
+ * gupnp_device_info_get_udn:(attributes org.gtk.Method.get_property=udn):
  * @info: A #GUPnPDeviceInfo
  *
  * Get the Unique Device Name of the device.
@@ -450,10 +503,10 @@ gupnp_device_info_get_udn (GUPnPDeviceInfo *info)
 }
 
 /**
- * gupnp_device_info_get_device_type:
+ * gupnp_device_info_get_device_type:(attributes org.gtk.Method.get_property=device-type):
  * @info: A #GUPnPDeviceInfo
  *
- * Get the UPnP device type.
+ * Get the UPnP device type of this #GUPnPDeviceInfo, e.g. `urn:schemas-upnp-org:device:InternetGatewayDevice:1`
  *
  * Returns: A constant string, or %NULL.
  **/
@@ -480,7 +533,8 @@ gupnp_device_info_get_device_type (GUPnPDeviceInfo *info)
  *
  * Get the friendly name of the device.
  *
- * Return value: A string, or %NULL. g_free() after use.
+ * Return value:(nullable)(transfer full):A newly allocated string containing the
+ * "friendly name" of the device, or %NULL if not available. g_free() after use.
  **/
 char *
 gupnp_device_info_get_friendly_name (GUPnPDeviceInfo *info)
@@ -501,7 +555,8 @@ gupnp_device_info_get_friendly_name (GUPnPDeviceInfo *info)
  *
  * Get the manufacturer of the device.
  *
- * Return value:(nullable)(transfer full): A string, or %NULL. g_free() after use.
+ * Return value:(nullable)(transfer full): A newly allocated string containing the
+ * manufacturer of the device, or %NULL if not available. g_free() after use.
  **/
 char *
 gupnp_device_info_get_manufacturer (GUPnPDeviceInfo *info)
@@ -670,7 +725,7 @@ gupnp_device_info_get_upc (GUPnPDeviceInfo *info)
  * @info: A #GUPnPDeviceInfo
  *
  * Get an URL pointing to the device's presentation page, for web-based
- * administration.
+ * administration, if available.
  *
  * Return value:(nullable)(transfer full): A string, or %NULL. g_free() after use.
  **/
@@ -722,12 +777,8 @@ icon_parse (xmlNode *element)
 static void
 icon_free (Icon *icon)
 {
-        if (icon->mime_type)
-                xmlFree (icon->mime_type);
-
-        if (icon->url)
-                xmlFree (icon->url);
-
+        g_clear_pointer (&icon->mime_type, xmlFree);
+        g_clear_pointer (&icon->url, xmlFree);
         g_slice_free (Icon, icon);
 }
 
@@ -752,7 +803,9 @@ icon_free (Icon *icon)
  * returned icon, or %NULL
  *
  * Get an URL pointing to the icon most closely matching the
- * given criteria, or %NULL. If @requested_mime_type is set, only icons with
+ * given criteria, or %NULL.
+ *
+ * If @requested_mime_type is set, only icons with
  * this mime type will be returned. If @requested_depth is set, only icons with
  * this or lower depth will be returned. If @requested_width and/or
  * @requested_height are set, only icons that are this size or smaller are
@@ -871,7 +924,7 @@ gupnp_device_info_get_icon_url (GUPnPDeviceInfo *info,
                         icon = l->data;
 
                         /* Look between icons with positive weight first */
-                        if (icon->weight >= 0) {
+                        if (icon && icon->weight >= 0) {
                                 if (!closest || icon->weight < closest->weight)
                                         closest = icon;
                         }
@@ -884,7 +937,8 @@ gupnp_device_info_get_icon_url (GUPnPDeviceInfo *info,
 
                         /* No icons with positive weight, look at ones with
                          * negative weight */
-                        if (!closest || icon->weight > closest->weight)
+                        if (!closest ||
+                            (icon && (icon->weight > closest->weight)))
                                 closest = icon;
                 }
         }
@@ -909,12 +963,15 @@ gupnp_device_info_get_icon_url (GUPnPDeviceInfo *info,
                         *height = icon->height;
 
                 if (icon->url) {
-                        SoupURI *uri;
+                        GUri *uri;
 
-                        uri = soup_uri_new_with_base (priv->url_base,
-                                                      (const char *) icon->url);
-                        ret = soup_uri_to_string (uri, FALSE);
-                        soup_uri_free (uri);
+                        uri = g_uri_parse_relative (priv->url_base,
+                                                    (const char *) icon->url,
+                                                    G_URI_FLAGS_NONE,
+                                                    NULL);
+                        ret = g_uri_to_string_partial (uri,
+                                                       G_URI_HIDE_PASSWORD);
+                        g_uri_unref (uri);
                 } else
                         ret = NULL;
         } else {
@@ -996,11 +1053,12 @@ resource_type_match (const char *query,
  * gupnp_device_info_list_dlna_device_class_identifier:
  * @info: A #GUPnPDeviceInfo
  *
- * Get a #GList of strings that represent the device class and version as
- * announced in the device description file using the &lt;dlna:X_DLNADOC&gt;
- * element.
+ * Get a list of strings that represent the device class and version as
+ * announced in the device description file using the `<dlna:X_DLNADOC>`
+ * element, e.g. `DMS-1.51`, `M-DMS-1.51` and so on.
+ *
  * Returns:(nullable)(transfer full) (element-type utf8): a #GList of newly allocated strings or
- * %NULL if the device description doesn't contain the &lt;dlna:X_DLNADOC&gt;
+ * %NULL if the device description doesn't contain any `<dlna:X_DLNADOC>`
  * element.
  *
  * Since: 0.20.4
@@ -1106,8 +1164,8 @@ gupnp_device_info_list_dlna_capabilities (GUPnPDeviceInfo *info)
  * This function provides generic access to the contents of arbitrary elements
  * in the device description file.
  *
- * Return value:(nullable)(transfer full): a newly allocated string or %NULL if the device
- *               description doesn't contain the given @element
+ * Return value:(nullable)(transfer full): a newly allocated string containing the
+ * requested value or %NULL if the device description doesn't contain the given @element
  *
  * Since: 0.14.0
  **/
@@ -1131,8 +1189,7 @@ gupnp_device_info_get_description_value (GUPnPDeviceInfo *info,
  * @info: A #GUPnPDeviceInfo
  *
  * Get a #GList of new objects implementing #GUPnPDeviceInfo
- * representing the devices directly contained in @info. The returned list
- * should be g_list_free()'d and the elements should be g_object_unref()'d.
+ * representing the devices directly contained in @info, excluding itself.
  *
  * Note that devices are not cached internally, so that every time you
  * call this function new objects are created. The application
@@ -1140,7 +1197,7 @@ gupnp_device_info_get_description_value (GUPnPDeviceInfo *info,
  * them.
  *
  * Return value:(nullable)(element-type GUPnP.DeviceInfo) (transfer full): a #GList of
- * new #GUPnPDeviceInfo objects.
+ * new #GUPnPDeviceInfo objects or %NULL if no devices are
  **/
 GList *
 gupnp_device_info_list_devices (GUPnPDeviceInfo *info)
@@ -1154,7 +1211,7 @@ gupnp_device_info_list_devices (GUPnPDeviceInfo *info)
 
         class = GUPNP_DEVICE_INFO_GET_CLASS (info);
 
-        g_return_val_if_fail (class->get_device, NULL);
+        g_return_val_if_fail (class->create_device_instance != NULL, NULL);
 
         devices = NULL;
 
@@ -1170,7 +1227,7 @@ gupnp_device_info_list_devices (GUPnPDeviceInfo *info)
                 if (!strcmp ("device", (char *) element->name)) {
                         GUPnPDeviceInfo *child;
 
-                        child = class->get_device (info, element);
+                        child = gupnp_device_info_create_device_instance (info, element);
                         devices = g_list_prepend (devices, child);
                 }
         }
@@ -1228,7 +1285,7 @@ gupnp_device_info_list_device_types (GUPnPDeviceInfo *info)
  * @info: A #GUPnPDeviceInfo
  * @type: The type of the device to be retrieved.
  *
- * Get the service with type @type directly contained in @info as
+ * Get the device with type @type directly contained in @info as
  * a new object implementing #GUPnPDeviceInfo, or %NULL if no such device
  * was found. The returned object should be unreffed when done.
  *
@@ -1252,7 +1309,7 @@ gupnp_device_info_get_device (GUPnPDeviceInfo *info,
 
         class = GUPNP_DEVICE_INFO_GET_CLASS (info);
 
-        g_return_val_if_fail (class->get_device, NULL);
+        g_return_val_if_fail (class->create_service_instance != NULL, NULL);
 
         device = NULL;
         priv = gupnp_device_info_get_instance_private (info);
@@ -1279,7 +1336,10 @@ gupnp_device_info_get_device (GUPnPDeviceInfo *info,
                                 continue;
 
                         if (resource_type_match (type, (char *) type_str))
-                                device = class->get_device (info, element);
+                                device =
+                                        gupnp_device_info_create_device_instance (
+                                                info,
+                                                element);
 
                         xmlFree (type_str);
 
@@ -1317,7 +1377,7 @@ gupnp_device_info_list_services (GUPnPDeviceInfo *info)
 
         class = GUPNP_DEVICE_INFO_GET_CLASS (info);
 
-        g_return_val_if_fail (class->get_service, NULL);
+        g_return_val_if_fail (class->create_service_instance, NULL);
 
         services = NULL;
 
@@ -1333,7 +1393,7 @@ gupnp_device_info_list_services (GUPnPDeviceInfo *info)
                 if (!strcmp ("service", (char *) element->name)) {
                         GUPnPServiceInfo *service;
 
-                        service = class->get_service (info, element);
+                        service = gupnp_device_info_create_service_instance (info, element);
                         services = g_list_prepend (services, service);
                 }
         }
@@ -1346,7 +1406,7 @@ gupnp_device_info_list_services (GUPnPDeviceInfo *info)
  * @info: A #GUPnPDeviceInfo
  *
  * Get a #GList of strings representing the types of the services
- * directly contained in @info.
+ * directly contained in @info, but not in its subdevices.
  *
  * Return value: (nullable)(element-type utf8) (transfer full): A #GList of strings. The
  * elements should be g_free()'d and the list should be g_list_free()'d.
@@ -1392,8 +1452,7 @@ gupnp_device_info_list_service_types (GUPnPDeviceInfo *info)
  * @type: The type of the service to be retrieved.
  *
  * Get the service with type @type directly contained in @info as a new object
- * implementing #GUPnPServiceInfo, or %NULL if no such device was found. The
- * returned object should be unreffed when done.
+ * implementing #GUPnPServiceInfo, or %NULL if no such device was found.
  *
  * Note that services are not cached internally, so that every time you call
  * this function a new object is created. The application must cache any used
@@ -1415,7 +1474,7 @@ gupnp_device_info_get_service (GUPnPDeviceInfo *info,
 
         class = GUPNP_DEVICE_INFO_GET_CLASS (info);
 
-        g_return_val_if_fail (class->get_service, NULL);
+        g_return_val_if_fail (class->create_service_instance, NULL);
 
         service = NULL;
 
@@ -1443,7 +1502,7 @@ gupnp_device_info_get_service (GUPnPDeviceInfo *info,
                                 continue;
 
                         if (resource_type_match (type, (char *) type_str))
-                                service = class->get_service (info, element);
+                                service = gupnp_device_info_create_service_instance (info, element);
 
                         xmlFree (type_str);
 
@@ -1466,12 +1525,171 @@ _gupnp_device_info_get_document (GUPnPDeviceInfo *info)
         return priv->doc;
 }
 
-void
-_gupnp_device_info_set_document (GUPnPDeviceInfo *info, GUPnPXMLDoc *doc)
+static void
+on_get_icon_async (GObject *source, GAsyncResult *res, gpointer user_data)
 {
-        GUPnPDeviceInfoPrivate *priv;
+        GTask *task = G_TASK (user_data);
+        GError *error = NULL;
 
-        priv = gupnp_device_info_get_instance_private (info);
+        GBytes *data = soup_session_send_and_read_finish (SOUP_SESSION (source),
+                                                          res,
+                                                          &error);
+        SoupMessage *message =
+                soup_session_get_async_result_message (SOUP_SESSION (source),
+                                                       res);
 
-        priv->doc = g_object_ref (doc);
+        if (error != NULL) {
+                g_task_return_error (task, error);
+        } else {
+                if (SOUP_STATUS_IS_SUCCESSFUL (
+                            soup_message_get_status (message))) {
+                        g_task_return_pointer (task,
+                                               g_steal_pointer (&data),
+                                               (GDestroyNotify) g_bytes_unref);
+                } else {
+                        g_bytes_unref (data);
+
+                        g_task_return_new_error (
+                                task,
+                                G_IO_ERROR,
+                                G_IO_ERROR_FAILED,
+                                "Unable to download icon: %s",
+                                soup_message_get_reason_phrase (message));
+                }
+        }
+
+        g_clear_pointer (&data, g_bytes_unref);
+        g_object_unref (task);
+}
+
+/**
+ * gupnp_device_info_get_icon_async:
+ * @info: A #GUPnPDeviceInfo
+ * @requested_mime_type: (nullable) (transfer none): The requested file
+ * format, or %NULL for any
+ * @requested_depth: The requested color depth, or -1 for any
+ * @requested_width: The requested width, or -1 for any
+ * @requested_height: The requested height, or -1 for any
+ * @prefer_bigger: %TRUE if a bigger, rather than a smaller icon should be
+ * returned if no exact match could be found
+ *
+ * Download the device icon matching the request parameters. For details on
+ * the lookup procedure, see [method@GUPnP.DeviceInfo.get_icon_url]
+ **/
+void
+gupnp_device_info_get_icon_async (GUPnPDeviceInfo *info,
+                                  const char *requested_mime_type,
+                                  int requested_depth,
+                                  int requested_width,
+                                  int requested_height,
+                                  gboolean prefer_bigger,
+                                  GCancellable *cancellable,
+                                  GAsyncReadyCallback callback,
+                                  gpointer user_data)
+{
+        g_return_if_fail (GUPNP_IS_DEVICE_INFO (info));
+
+        GTask *task = g_task_new (info, cancellable, callback, user_data);
+        char *mime = NULL;
+        int depth = -1;
+        int width = -1;
+        int height = -1;
+
+        char *url = gupnp_device_info_get_icon_url (info,
+                                                    requested_mime_type,
+                                                    requested_depth,
+                                                    requested_width,
+                                                    requested_height,
+                                                    prefer_bigger,
+                                                    &mime,
+                                                    &depth,
+                                                    &width,
+                                                    &height);
+
+        if (url == NULL) {
+                g_task_return_pointer (task, NULL, NULL);
+                g_object_unref (task);
+                g_clear_pointer (&mime, g_free);
+
+                return;
+        }
+
+        GUPnPContext *context = gupnp_device_info_get_context (info);
+        SoupSession *session = gupnp_context_get_session (context);
+        GUri *rewritten_url = gupnp_context_rewrite_uri_to_uri (context, url);
+        g_free (url);
+
+        GetIconData *data = g_new0 (GetIconData, 1);
+        data->mime = g_steal_pointer (&mime);
+        data->depth = depth;
+        data->width = width;
+        data->height = height;
+        g_task_set_task_data (task, data, (GDestroyNotify) get_icon_data_free);
+
+        SoupMessage *message =
+                soup_message_new_from_uri (SOUP_METHOD_GET, rewritten_url);
+        g_uri_unref (rewritten_url);
+
+        soup_session_send_and_read_async (session,
+                                          message,
+                                          G_PRIORITY_DEFAULT_IDLE,
+                                          g_task_get_cancellable (task),
+                                          on_get_icon_async,
+                                          task);
+        g_object_unref (message);
+        g_clear_pointer (&mime, g_free);
+}
+
+/**
+ * gupnp_device_info_get_icon_finish:
+ * @info: A #GUPnPDeviceInfo
+ * @res: A GAsyncResult
+ * @mime: (out) (optional): The location where to store the the format
+ * of the returned icon, or %NULL. The returned string should be freed after
+ * use
+ * @depth: (out) (optional):  The location where to store the depth of the
+ * returned icon, or %NULL
+ * @width: (out) (optional): The location where to store the width of the
+ * returned icon, or %NULL
+ * @height: (out) (optional): The location where to store the height of the
+ * returned icon, or %NULL
+ * @error: A location for GError or NULL
+ *
+ * Returns: (transfer full): A GBytes contaning the icon or NULL on either
+ * error or no matching icon was found.
+ */
+GBytes *
+gupnp_device_info_get_icon_finish (GUPnPDeviceInfo *info,
+                                   GAsyncResult *res,
+                                   char **mime,
+                                   int *depth,
+                                   int *width,
+                                   int *height,
+                                   GError **error)
+{
+        g_return_val_if_fail (GUPNP_IS_DEVICE_INFO (info), NULL);
+        g_return_val_if_fail (g_task_is_valid (res, info), NULL);
+
+        GBytes *data = g_task_propagate_pointer (G_TASK (res), error);
+        GetIconData *icon_data = g_task_get_task_data (G_TASK (res));
+
+        if (mime != NULL) {
+                *mime = (icon_data == NULL)
+                                ? NULL
+                                : g_steal_pointer (&(icon_data->mime));
+        }
+
+        if (depth != NULL) {
+                *depth = (icon_data == NULL) ? -1 : icon_data->depth;
+        }
+
+        if (width != NULL) {
+                *width = (icon_data == NULL) ? -1 : icon_data->width;
+        }
+
+        if (height != NULL) {
+                *height = (icon_data == NULL) ? -1 : icon_data->height;
+        }
+
+        return data;
 }
