@@ -89,13 +89,12 @@ test_bgo_696762_on_browse_call (G_GNUC_UNUSED GUPnPService *service,
 
     g_assert (node != NULL);
     g_assert_cmpstr ((const char *) node->name, ==, "SortCriteria");
-    node = node->next;
     gupnp_service_action_return_success (action);
 }
 
 static void
-test_bgo_696762_on_browse (G_GNUC_UNUSED GUPnPServiceProxy       *proxy,
-                           G_GNUC_UNUSED GUPnPServiceProxyAction *action,
+test_bgo_696762_on_browse (G_GNUC_UNUSED GObject       *source,
+                           G_GNUC_UNUSED GAsyncResult *res,
                            gpointer                               user_data)
 {
     TestServiceProxyData *data = (TestServiceProxyData *) user_data;
@@ -163,15 +162,16 @@ test_bgo_690400_query_variable (GUPnPService *service,
 }
 
 static gboolean
-test_on_timeout (G_GNUC_UNUSED gpointer user_data)
+test_on_timeout (gpointer user_data)
 {
-    g_assert_not_reached ();
+        g_print ("Timeout in %s\n", (const char *) user_data);
+        g_assert_not_reached ();
 
-    return FALSE;
+        return FALSE;
 }
 
 static void
-test_run_loop (GMainLoop *loop)
+test_run_loop (GMainLoop *loop, const char *name)
 {
         guint timeout_id = 0;
         int timeout = 2;
@@ -188,11 +188,19 @@ test_run_loop (GMainLoop *loop)
         g_source_remove (timeout_id);
 }
 
+static gboolean
+delayed_loop_quitter (gpointer user_data)
+{
+        g_main_loop_quit (user_data);
+        return G_SOURCE_REMOVE;
+}
+
 /* Test if a call on a service proxy keeps argument order */
 static void
 test_bgo_696762 (void)
 {
     GUPnPContext *context = NULL;
+    GUPnPContext *server_context = NULL;
     GError *error = NULL;
     GUPnPControlPoint *cp = NULL;
     GUPnPRootDevice *rd;
@@ -205,6 +213,10 @@ test_bgo_696762 (void)
     g_assert_no_error (error);
     g_assert (context != NULL);
 
+    server_context = create_context (0, &error);
+    g_assert_no_error (error);
+    g_assert (server_context != NULL);
+
     cp = gupnp_control_point_new (context,
                                   "urn:test-gupnp-org:service:TestService:1");
 
@@ -216,7 +228,7 @@ test_bgo_696762 (void)
                       &data);
 
 
-    rd = gupnp_root_device_new (context, "TestDevice.xml", DATA_PATH, &error);
+    rd = gupnp_root_device_new (server_context, "TestDevice.xml", DATA_PATH, &error);
     g_assert_no_error (error);
     g_assert (rd != NULL);
     gupnp_root_device_set_available (rd, TRUE);
@@ -227,30 +239,58 @@ test_bgo_696762 (void)
                       G_CALLBACK (test_bgo_696762_on_browse_call),
                       &data);
 
-    test_run_loop (data.loop);
+    test_run_loop (data.loop, g_test_get_path ());
     g_assert (data.proxy != NULL);
 
     G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-    gupnp_service_proxy_begin_action (data.proxy,
-                                      "Browse",
-                                      test_bgo_696762_on_browse,
-                                      &data,
-                                      "ObjectID", G_TYPE_STRING, "0",
-                                      "BrowseFlag", G_TYPE_STRING, "BrowseDirectChildren",
-                                      "Filter", G_TYPE_STRING, "res,dc:date,res@size",
-                                      "StartingIndex", G_TYPE_UINT, 0,
-                                      "RequestedCount", G_TYPE_UINT, 0,
-                                      "SortCriteria", G_TYPE_STRING, "",
-                                      NULL);
-    G_GNUC_END_IGNORE_DEPRECATIONS
 
-    test_run_loop (data.loop);
+    GUPnPServiceProxyAction *a =
+            gupnp_service_proxy_action_new ("Browse",
+                                            "ObjectID",
+                                            G_TYPE_STRING,
+                                            "0",
+                                            "BrowseFlag",
+                                            G_TYPE_STRING,
+                                            "BrowseDirectChildren",
+                                            "Filter",
+                                            G_TYPE_STRING,
+                                            "res,dc:date,res@size",
+                                            "StartingIndex",
+                                            G_TYPE_UINT,
+                                            0,
+                                            "RequestedCount",
+                                            G_TYPE_UINT,
+                                            0,
+                                            "SortCriteria",
+                                            G_TYPE_STRING,
+                                            "",
+                                            NULL);
 
-    g_main_loop_unref (data.loop);
+    gupnp_service_proxy_call_action_async (data.proxy,
+                                           a,
+                                           NULL,
+                                           test_bgo_696762_on_browse,
+                                           &data);
+
+
+    test_run_loop (data.loop, g_test_get_path ());
+
+    gupnp_service_proxy_action_unref (a);
+
+    // tear-down
+    g_object_unref (info);
     g_object_unref (data.proxy);
+
     g_object_unref (cp);
     g_object_unref (rd);
+    g_object_unref (server_context);
     g_object_unref (context);
+
+    // Make sure the source teardown handlers get run so we don't confuse valgrind
+    g_timeout_add (500, (GSourceFunc) delayed_loop_quitter, data.loop);
+    g_main_loop_run (data.loop);
+
+    g_main_loop_unref (data.loop);
 }
 
 /* Test that proxies created by ResourceFactory are of the GType
@@ -274,7 +314,8 @@ test_bgo_678701 (void)
     g_assert_no_error (error);
     g_assert (context != NULL);
 
-    factory = gupnp_resource_factory_get_default ();
+    // Do not pollute the default factory with this test
+    factory = gupnp_resource_factory_new ();
     gupnp_resource_factory_register_resource_proxy_type (factory,
                                                          "urn:test-gupnp-org:service:TestService:1",
                                                          test_bgo_678701_service_get_type ());
@@ -287,15 +328,17 @@ test_bgo_678701 (void)
     g_assert (rd != NULL);
     gupnp_root_device_set_available (rd, TRUE);
 
-    cp = gupnp_control_point_new (context,
-                                  "urn:test-gupnp-org:device:TestDevice:1");
+    cp = gupnp_control_point_new_full (
+            context,
+            factory,
+            "urn:test-gupnp-org:device:TestDevice:1");
     gssdp_resource_browser_set_active (GSSDP_RESOURCE_BROWSER (cp), TRUE);
     g_signal_connect (G_OBJECT (cp),
                       "device-proxy-available",
                       G_CALLBACK (test_bgo_678701_on_dp_available),
                       &data);
 
-    test_run_loop (data.loop);
+    test_run_loop (data.loop, g_test_get_path ());
     g_assert (data.proxy != NULL);
 
     info = gupnp_device_info_get_service (GUPNP_DEVICE_INFO (data.proxy),
@@ -306,11 +349,20 @@ test_bgo_678701 (void)
                                           "urn:test-gupnp-org:device:TestSubDevice:1");
     g_assert_cmpstr(G_OBJECT_TYPE_NAME (dev_info), ==, "TestBgo678701Device");
 
-    g_main_loop_unref (data.loop);
     g_object_unref (data.proxy);
     g_object_unref (cp);
+    g_object_unref (info);
+    g_object_unref (dev_info);
     g_object_unref (rd);
     g_object_unref (context);
+    g_object_unref (factory);
+
+    // Make sure the source teardown handlers get run so we don't confuse valgrind
+    g_timeout_add (500, (GSourceFunc) delayed_loop_quitter, data.loop);
+    g_main_loop_run (data.loop);
+
+    g_main_loop_unref (data.loop);
+
 }
 
 /* Test that removing a notify-callback from the callback itself works
@@ -349,7 +401,7 @@ test_bgo_690400 (void)
                       G_CALLBACK (test_bgo_690400_query_variable), NULL);
     gupnp_root_device_set_available (rd, TRUE);
 
-    test_run_loop (data.loop);
+    test_run_loop (data.loop, "690400 - waiting for query_variable");
     g_assert (data.proxy != NULL);
 
     gupnp_service_proxy_add_notify (data.proxy,
@@ -365,14 +417,19 @@ test_bgo_690400 (void)
 
     gupnp_service_proxy_set_subscribed (data.proxy, TRUE);
 
-    test_run_loop (data.loop);
+    test_run_loop (data.loop, "690400 - waiting for event");
 
-    g_main_loop_unref (data.loop);
     g_object_unref (data.proxy);
     g_object_unref (cp);
     g_object_unref (service);
     g_object_unref (rd);
     g_object_unref (context);
+
+    // Make sure the source teardown handlers get run so we don't confuse valgrind
+    g_timeout_add (500, (GSourceFunc) delayed_loop_quitter, data.loop);
+    g_main_loop_run (data.loop);
+
+    g_main_loop_unref (data.loop);
 }
 
 /* Test that correct icons are returned for various size requests.
@@ -531,143 +588,6 @@ test_ggo_24 (void)
                 validate_host_header ("[fe80::01%eth0]", "fe80::acab", 4711));
 }
 
-/*
- * Test that the legacy async _end_action calls still work
- *
- * https://gitlab.gnome.org/GNOME/gupnp/-/issues/58
- */
-
-static void
-test_ggo_58_on_ping (GUPnPServiceProxy *proxy,
-                     GUPnPServiceProxyAction *action,
-                     gpointer user_data)
-{
-        TestServiceProxyData *data = (TestServiceProxyData *) user_data;
-
-        g_main_loop_quit (data->loop);
-}
-
-static void
-test_ggo_58_on_ping_call (GUPnPService *service,
-                          GUPnPServiceAction *action,
-                          gpointer user_data)
-{
-        gupnp_service_action_return_success (action);
-}
-
-static void
-test_ggo_58 ()
-{
-        GUPnPContext *context = NULL;
-        GError *error = NULL;
-        GUPnPControlPoint *cp = NULL;
-        GUPnPRootDevice *rd;
-        TestServiceProxyData data = { NULL, NULL };
-        GUPnPServiceInfo *info = NULL;
-
-        data.loop = g_main_loop_new (NULL, FALSE);
-
-        context = create_context (0, &error);
-        g_assert_no_error (error);
-        g_assert (context != NULL);
-
-        cp = gupnp_control_point_new (
-                context,
-                "urn:test-gupnp-org:service:TestService:1");
-
-        gssdp_resource_browser_set_active (GSSDP_RESOURCE_BROWSER (cp), TRUE);
-
-        g_signal_connect (G_OBJECT (cp),
-                          "service-proxy-available",
-                          G_CALLBACK (test_on_sp_available),
-                          &data);
-
-
-        rd = gupnp_root_device_new (context,
-                                    "TestDevice.xml",
-                                    DATA_PATH,
-                                    &error);
-        g_assert_no_error (error);
-        g_assert (rd != NULL);
-        gupnp_root_device_set_available (rd, TRUE);
-        info = gupnp_device_info_get_service (
-                GUPNP_DEVICE_INFO (rd),
-                "urn:test-gupnp-org:service:TestService:1");
-        g_signal_connect (G_OBJECT (info),
-                          "action-invoked::Ping",
-                          G_CALLBACK (test_ggo_58_on_ping_call),
-                          &data);
-
-        test_run_loop (data.loop);
-        g_assert (data.proxy != NULL);
-
-        G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-        GUPnPServiceProxyAction *action =
-                gupnp_service_proxy_begin_action (data.proxy,
-                                                  "Ping",
-                                                  test_ggo_58_on_ping,
-                                                  &data,
-                                                  NULL);
-
-        test_run_loop (data.loop);
-
-        gboolean success = gupnp_service_proxy_end_action (data.proxy,
-                                                           action,
-                                                           &error,
-                                                           NULL);
-
-        g_assert (success);
-        g_assert_no_error (error);
-
-        action = gupnp_service_proxy_begin_action (data.proxy,
-                                                   "Ping",
-                                                   test_ggo_58_on_ping,
-                                                   &data,
-                                                   NULL);
-
-        test_run_loop (data.loop);
-
-        GHashTable *result_hash = g_hash_table_new (g_str_hash, g_str_equal);
-
-        success = gupnp_service_proxy_end_action_hash (data.proxy,
-                                                       action,
-                                                       result_hash,
-                                                       &error);
-        g_hash_table_destroy (result_hash);
-
-        g_assert (success);
-        g_assert_no_error (error);
-
-        action = gupnp_service_proxy_begin_action (data.proxy,
-                                                   "Ping",
-                                                   test_ggo_58_on_ping,
-                                                   &data,
-                                                   NULL);
-
-        test_run_loop (data.loop);
-
-        GList *result_list = NULL;
-        success = gupnp_service_proxy_end_action_list (data.proxy,
-                                                       action,
-                                                       NULL,
-                                                       NULL,
-                                                       &result_list,
-                                                       &error);
-
-        g_assert (success);
-        g_assert_no_error (error);
-
-        G_GNUC_END_IGNORE_DEPRECATIONS
-
-        g_object_unref (info);
-        g_object_unref (data.proxy);
-        g_object_unref (cp);
-        g_object_unref (rd);
-        g_object_unref (context);
-
-        g_main_loop_unref (data.loop);
-}
-
 void
 test_ggo_42 ()
 {
@@ -691,31 +611,49 @@ test_ggo_42 ()
                 GUPNP_DEVICE_INFO (rd),
                 "urn:test-gupnp-org:service:TestService:1");
 
+        GMainLoop *loop = g_main_loop_new (NULL, FALSE);
         g_object_unref (rd);
+        g_timeout_add (500, (GSourceFunc) delayed_loop_quitter, loop);
+        g_main_loop_run (loop);
+
         g_object_unref (info);
+
+        // Make sure the source teardown handlers get run so we don't confuse valgrind
+        g_timeout_add (500, (GSourceFunc) delayed_loop_quitter, loop);
+        g_main_loop_run (loop);
+
+        g_object_unref (context);
+
+        // Make sure the source teardown handlers get run so we don't confuse valgrind
+        g_timeout_add (500, (GSourceFunc) delayed_loop_quitter, loop);
+        g_main_loop_run (loop);
+
+        g_main_loop_unref (loop);
 }
 
 void
 on_control (SoupServer *server,
-            SoupMessage *msg,
+            SoupServerMessage *msg,
             const char *path,
             GHashTable *query,
-            SoupClientContext *ctx,
             gpointer user_data)
 {
-        g_print ("Method: %s\n", msg->method);
-        soup_message_set_status (msg, SOUP_STATUS_METHOD_NOT_ALLOWED);
+        SoupMessageBody *b = soup_server_message_get_response_body (msg);
+
+        g_print ("On Control\n");
+
+        soup_message_body_append (b, SOUP_MEMORY_COPY, "<html>\0", 7);
+        soup_server_message_set_status (msg, SOUP_STATUS_METHOD_NOT_ALLOWED, NULL);
 }
 
 void
 on_event (SoupServer *server,
-          SoupMessage *msg,
+          SoupServerMessage *msg,
           const char *path,
           GHashTable *query,
-          SoupClientContext *ctx,
           gpointer user_data)
 {
-        soup_message_set_status (msg, SOUP_STATUS_OK);
+        soup_server_message_set_status (msg, SOUP_STATUS_OK, NULL);
 }
 
 typedef struct {
@@ -736,6 +674,8 @@ on_proxy (GUPnPControlPoint *cp, GUPnPServiceProxy *p, gpointer user_data)
 void
 on_ping (GObject *object, GAsyncResult *res, gpointer user_data)
 {
+        g_print ("On PIng\n");
+
         TestGGO63Data *d = (TestGGO63Data *) user_data;
         GError *error = NULL;
 
@@ -745,10 +685,7 @@ on_ping (GObject *object, GAsyncResult *res, gpointer user_data)
                         res,
                         &error);
 
-        g_assert_nonnull (action);
-        g_assert_no_error (error);
-
-        gupnp_service_proxy_action_get_result (action, &error, NULL);
+        g_assert_null (action);
         g_assert_error (error, GUPNP_SERVER_ERROR, GUPNP_SERVER_ERROR_OTHER);
 
         g_error_free (error);
@@ -771,10 +708,10 @@ test_ggo_63 ()
         GSSDPResourceGroup *rg =
                 gssdp_resource_group_new (GSSDP_CLIENT (context));
 
-        char *server_uri = soup_uri_to_string (uris->data, FALSE);
+        char *server_uri = g_uri_to_string (uris->data);
         char *resource_path = g_strconcat (server_uri, "TestDevice.xml", NULL);
         g_free (server_uri);
-        g_slist_free_full (uris, (GDestroyNotify) soup_uri_free);
+        g_slist_free_full (uris, (GDestroyNotify) g_uri_unref);
 
         gssdp_resource_group_add_resource_simple (rg,
                                                   "upnp:rootdevice",
@@ -845,6 +782,12 @@ test_ggo_63 ()
         g_object_unref (cp);
         g_object_unref (rg);
         g_object_unref (context);
+
+
+        // Make sure the source teardown handlers get run so we don't confuse valgrind
+        g_timeout_add (500, (GSourceFunc) delayed_loop_quitter, d.loop);
+        g_main_loop_run (d.loop);
+
         g_main_loop_unref (d.loop);
 }
 
@@ -888,7 +831,7 @@ test_ggo_60_no_crash ()
         g_assert_no_error (error);
         g_assert (rd != NULL);
         gupnp_root_device_set_available (rd, TRUE);
-        test_run_loop (data.loop);
+        test_run_loop (data.loop, g_test_get_path ());
         g_assert (data.proxy != NULL);
 
         // We just use the default of "Action not implemented" response, since
@@ -902,150 +845,20 @@ test_ggo_60_no_crash ()
                                                test_ggo_60_call_ready,
                                                data.loop);
 
-        test_run_loop (data.loop);
+        test_run_loop (data.loop, g_test_get_path());
         gupnp_service_proxy_action_unref (action);
         g_object_unref (data.proxy);
         g_object_unref (rd);
         g_object_unref (cp);
         g_object_unref (context);
-        g_main_loop_unref (data.loop);
-}
 
-static void
-test_ggo_67_on_ping (GUPnPServiceProxy *proxy,
-                     GUPnPServiceProxyAction *action,
-                     gpointer user_data)
-{
-        TestServiceProxyData *data = (TestServiceProxyData *) user_data;
-
-        g_assert_nonnull (proxy);
-        g_assert_nonnull (action);
-        g_assert_nonnull (user_data);
-
-        g_main_loop_quit (data->loop);
-}
-
-static void
-test_ggo_67_on_ping_call (GUPnPService *service,
-                          GUPnPServiceAction *action,
-                          gpointer user_data)
-{
-        gupnp_service_action_return_success (action);
-}
-
-// Verbatim copy of 58, just with a different call-back
-static void
-test_ggo_67 ()
-{
-        GUPnPContext *context = NULL;
-        GError *error = NULL;
-        GUPnPControlPoint *cp = NULL;
-        GUPnPRootDevice *rd;
-        TestServiceProxyData data = { NULL, NULL };
-        GUPnPServiceInfo *info = NULL;
-
-        data.loop = g_main_loop_new (NULL, FALSE);
-
-        context = create_context (0, &error);
-        g_assert_no_error (error);
-        g_assert (context != NULL);
-
-        cp = gupnp_control_point_new (
-                context,
-                "urn:test-gupnp-org:service:TestService:1");
-
-        gssdp_resource_browser_set_active (GSSDP_RESOURCE_BROWSER (cp), TRUE);
-
-        g_signal_connect (G_OBJECT (cp),
-                          "service-proxy-available",
-                          G_CALLBACK (test_on_sp_available),
-                          &data);
-
-
-        rd = gupnp_root_device_new (context,
-                                    "TestDevice.xml",
-                                    DATA_PATH,
-                                    &error);
-        g_assert_no_error (error);
-        g_assert (rd != NULL);
-        gupnp_root_device_set_available (rd, TRUE);
-        info = gupnp_device_info_get_service (
-                GUPNP_DEVICE_INFO (rd),
-                "urn:test-gupnp-org:service:TestService:1");
-        g_signal_connect (G_OBJECT (info),
-                          "action-invoked::Ping",
-                          G_CALLBACK (test_ggo_67_on_ping_call),
-                          &data);
-
-        test_run_loop (data.loop);
-        g_assert (data.proxy != NULL);
-
-        G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-                GUPnPServiceProxyAction *action =
-                gupnp_service_proxy_begin_action (data.proxy,
-                                                  "Ping",
-                                                  test_ggo_67_on_ping,
-                                                  &data,
-                                                  NULL);
-
-        test_run_loop (data.loop);
-
-        gboolean success = gupnp_service_proxy_end_action (data.proxy,
-                                                           action,
-                                                           &error,
-                                                           NULL);
-
-        g_assert (success);
-        g_assert_no_error (error);
-
-        action = gupnp_service_proxy_begin_action (data.proxy,
-                                                   "Ping",
-                                                   test_ggo_67_on_ping,
-                                                   &data,
-                                                   NULL);
-
-        test_run_loop (data.loop);
-
-        GHashTable *result_hash = g_hash_table_new (g_str_hash, g_str_equal);
-
-        success = gupnp_service_proxy_end_action_hash (data.proxy,
-                                                       action,
-                                                       result_hash,
-                                                       &error);
-        g_hash_table_destroy (result_hash);
-
-        g_assert (success);
-        g_assert_no_error (error);
-
-        action = gupnp_service_proxy_begin_action (data.proxy,
-                                                   "Ping",
-                                                   test_ggo_67_on_ping,
-                                                   &data,
-                                                   NULL);
-
-        test_run_loop (data.loop);
-
-        GList *result_list = NULL;
-        success = gupnp_service_proxy_end_action_list (data.proxy,
-                                                       action,
-                                                       NULL,
-                                                       NULL,
-                                                       &result_list,
-                                                       &error);
-
-        g_assert (success);
-        g_assert_no_error (error);
-
-        G_GNUC_END_IGNORE_DEPRECATIONS
-
-                        g_object_unref (info);
-        g_object_unref (data.proxy);
-        g_object_unref (cp);
-        g_object_unref (rd);
-        g_object_unref (context);
+        // Make sure the source teardown handlers get run so we don't confuse valgrind
+        g_timeout_add (500, (GSourceFunc) delayed_loop_quitter, data.loop);
+        g_main_loop_run (data.loop);
 
         g_main_loop_unref (data.loop);
 }
+
 int
 main (int argc, char *argv[]) {
     g_test_init (&argc, &argv, NULL);
@@ -1055,11 +868,9 @@ main (int argc, char *argv[]) {
     g_test_add_func ("/bugs/bgo/722696", test_bgo_722696);
     g_test_add_func ("/bugs/bgo/743233", test_bgo_743233);
     g_test_add_func ("/bugs/ggo/24", test_ggo_24);
-    g_test_add_func ("/bugs/ggo/58", test_ggo_58);
     g_test_add_func ("/bugs/ggo/42", test_ggo_42);
     g_test_add_func ("/bugs/ggo/63", test_ggo_63);
-    g_test_add_func ("/bugs/ggo/60/no-crash", test_ggo_60_no_crash);
-    g_test_add_func ("/bugs/ggo/67", test_ggo_67);
+    g_test_add_func ("/bugs/ggo/60", test_ggo_60_no_crash);
 
     return g_test_run ();
 }
